/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package com.myeis.j2c.formathandler;

import java.beans.BeanInfo;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Method;
import java.util.Hashtable;

import javax.wsdl.Binding;
import javax.wsdl.Definition;
import javax.xml.namespace.QName;
/*
* The class implementing FormatHandlerGenerator interface that 
* provides view for generating a Custom Format Handler for a particular XSD Type or Element.
* @param <code>generationPackage</code> The fully qualified package that the generator should use to generate its classes in.
* @param <code>beanClass</code> The fully qualified name of the Java bean class that represents the XSD type (or element) for which a format handler is being generated.
* @param <code>aDefinition</code> The WSDL definition from which the XSD type is derived.
* @param <code>aBinding</code> The WSDL binding element that contains the type mapping for the XSD type that is being used.
* @param <code>anEncoding</code> The <i>encoding</i> attribute off the TypeMapping element associated with the format handler to be generated
* @param <code>aStyle</code> The <i>style</i> attribute off the TypeMapping element associated with the format handler to be generated
* @param <code>xsdQname</code> The QName of the XSD element(or type) for which the format handler is being generated
* @param <code>elementType</code> The type of element represented by the <code>xsdQname</code> parameter. This is either <code>CustomFormatHandlerGenerator.XSD_TYPE</code> or <code>CustomFormatHandlerGenerator.XSD_ELEMENT</code>
* @return a <code>HashTable</code> that contains a set of  <code>java.io.InputStream</code> s. Each input stream contains the generated code for a particular
* class generated by the format handler generator. The number of input streams returned depends on the number of classes that are generated by the format handler
* generator.  The keys of the HashTable must be the name of each class (unqualified) that is represented by its corresponding InputStream value. The class name keys
* are unqualified because they must all be in the <code>generationPackage</code> that is passed in as an input parameter.
*/

import org.apache.wsif.WSIFException;
import org.apache.wsif.providers.jca.toolplugin.FormatHandlerGenerator;
import org.apache.wsif.util.WSIFUtils;

public class MyEISFormatHandlerGenerator implements FormatHandlerGenerator {

	/**
	 * @see org.apache.wsif.providers.jca.toolplugin.FormatHandlerGenerator#generate(String, String, Definition, Binding, String, String, QName, int, Object)
	 */
	public Hashtable generate(
		String generationPackage,
		String beanClass,
		javax.wsdl.Definition aDefinition,
		javax.wsdl.Binding aBinding,
		String anEncoding,
		String aStyle,
		javax.xml.namespace.QName xsdQname,
		int elementType,
		Object generationContext) throws WSIFException
		{

		if (!anEncoding.equals("myeis")) {
			System.err.println("Encoding: " + anEncoding);
			return null;
		}

		ClassLoader threadClassLoader = Thread.currentThread().getContextClassLoader();
		String definitionPackageName = WSIFUtils.getPackageNameFromNamespaceURI(aDefinition.getTargetNamespace());
		definitionPackageName = definitionPackageName.replace('.','/');
		String metaDataFile = "com/myeis/services/MyEISFormatHandlerGenMetaData.eis";
		InputStream metaDataStream = threadClassLoader.getResourceAsStream(metaDataFile);
		String metaDatafield =  null;
		
		if (metaDataStream != null) {
			try {
				//Read in the contents of the stream
				InputStreamReader reader = new InputStreamReader(metaDataStream);
				char buffer[] = new char[1024];
				StringBuffer data = new StringBuffer();
				int length;
				length = reader.read(buffer);
				while (length > 0) {
					data.append(buffer, 0, length);
					length = reader.read(buffer);
				}
				metaDatafield = data.toString();
			} catch (IOException e) {
				//Error occured reading metadata
				throw new WSIFException("Error occurred reading metadata needed for format handler generator!",e);
			}

		}

		String beanClassName = beanClass.substring(beanClass.lastIndexOf(".") + 1, beanClass.length());
		String beanClassField = "field" + beanClassName;
		String formatHandlerClassName = beanClassName + "FormatHandler";

		StringBuffer buffer = new StringBuffer();

		buffer.append("package " + generationPackage + ";\n");
		buffer.append(importsGen);
		buffer.append("public class " + beanClassName + "FormatHandler implements WSIFFormatHandler_JCA {\n");

		buffer.append("private " + beanClassName + " " + beanClassField + " = null;\n");
		
		if(metaDatafield!=null && metaDatafield.length()>0){
			buffer.append("private static String META_DATA_A = \""+ metaDatafield+"\";\n");
		}
		

		buffer.append("public void read(InputStream inputStream) throws IOException {\n");
		buffer.append("	try {\n");
		buffer.append("		ObjectInputStream objectInputStream = new ObjectInputStream(inputStream);\n");
		buffer.append("		Object obj = (Object)objectInputStream.readObject();\n");
		buffer.append("		if(obj instanceof "+beanClassName+"){\n");
		buffer.append("			this."+ beanClassField +" = ("+ beanClassName + ")obj;\n");
		buffer.append("		}else if (obj instanceof com.myeis.services.internal."+beanClassName+"DataObject){\n");
		buffer.append("			com.myeis.services.internal."+ beanClassName + "DataObject a"+ beanClassName + " = (com.myeis.services.internal."+ beanClassName + "DataObject)obj;\n");
		buffer.append("			this."+ beanClassField +" = new "+ beanClassName +"();\n");
		
		try {
			Class bean =
				Class.forName("com.myeis.services.internal." + beanClassName+"DataObject");
			BeanInfo bInfo = Introspector.getBeanInfo(bean);
			if (bInfo != null) {
				PropertyDescriptor properties[] =
					bInfo.getPropertyDescriptors();
				for (int i = 0; i < properties.length; i++) {
					Method writer = properties[i].getWriteMethod();
					Method reader = properties[i].getReadMethod();
					if (reader != null && writer != null) {
						String setter = writer.getName();
						String getter = reader.getName();
						buffer.append(
							"			this."
								+ beanClassField
								+ "."
								+ setter
								+ "(a"
								+ beanClassName
								+ "."
								+ getter
								+ "());\n");
					}
				}
			}
		} catch (Exception e) {
//			throw new WSIFException(
//				"Error occurred introspecting bean properties in format handler generator!",
//				e);

			buffer.append(" String ex ="+e+";\n");
		}
	
		buffer.append("		}\n");
		buffer.append("	} catch (ClassNotFoundException e) {\n");
		buffer.append("		e.printStackTrace();\n");
		buffer.append("		throw new IOException(\"class not found !\");\n");
		buffer.append("	}		\n");
		buffer.append("}\n\n");
		buffer.append("public void write(OutputStream outputStream) throws IOException {\n");

		buffer.append("    ObjectOutputStream objectOutputStream = new ObjectOutputStream(outputStream);\n");
		buffer.append("    objectOutputStream.writeObject(this." + beanClassField + ");\n");
		buffer.append("    objectOutputStream.flush();\n");
		buffer.append("}\n\n");

		buffer.append("public void setObjectPart(Object objectPart) {\n");
		buffer.append("	this." + beanClassField + " = (" + beanClassName + ")objectPart;\n");
		buffer.append("}\n\n");
		buffer.append("public Object getObjectPart() {\n");
		buffer.append("	return this." + beanClassField + ";\n");
		buffer.append("}\n\n");

		buffer.append("public Object getObjectPart(Class sourceClass) {\n");
		buffer.append("	try{\n");
		buffer.append("		if (sourceClass.isAssignableFrom(Class.forName(\"javax.xml.transform.sax.SAXSource\"))){\n");
		buffer.append("			SAXSource saxSource = new SAXSource(new InputSource());\n");
		buffer.append("			XMLReader reader = new ContentReader();\n");
		buffer.append("			saxSource.setXMLReader(reader);\n");
		buffer.append("			return saxSource;\n");
		buffer.append("		}\n");
		buffer.append("	}\n");
		buffer.append("	catch (Throwable e) {\n");
		buffer.append("	}\n");

		buffer.append("	return null;\n");
		buffer.append("}\n\n");

		buffer.append(defaultMethodsGen);

		writeContentReader(buffer, beanClassField);
		buffer.append("}"); // end of class

		ByteArrayInputStream inputStream = new ByteArrayInputStream(buffer.toString().getBytes());
		Hashtable returnTable = new Hashtable();
		returnTable.put(formatHandlerClassName, inputStream);
		return returnTable;

	}

	private void writeContentReader(StringBuffer buffer, String beanClassField) {
		buffer.append("public class ContentReader implements org.xml.sax.XMLReader {\n");
		buffer.append("	private ContentHandler fieldContentHandler = null;\n");

		buffer.append("	public org.xml.sax.ContentHandler getContentHandler() {return null;}\n");
		buffer.append("	public org.xml.sax.DTDHandler getDTDHandler() {return null;}\n");
		buffer.append("	public org.xml.sax.EntityResolver getEntityResolver() {return null;}\n");
		buffer.append("	public org.xml.sax.ErrorHandler getErrorHandler() {return null;}\n");
		buffer.append("	public boolean getFeature(String name) throws org.xml.sax.SAXNotSupportedException, org.xml.sax.SAXNotRecognizedException {return false;}\n");
		buffer.append("	public Object getProperty(String name) throws org.xml.sax.SAXNotSupportedException, org.xml.sax.SAXNotRecognizedException {return null;}\n");
		buffer.append("	public void setDTDHandler(org.xml.sax.DTDHandler handler) {}\n");
		buffer.append("	public void setEntityResolver(org.xml.sax.EntityResolver resolver) {}\n");
		buffer.append("	public void setErrorHandler(org.xml.sax.ErrorHandler handler) {}\n");
		buffer.append("	public void setFeature(String name, boolean value) throws org.xml.sax.SAXNotSupportedException, org.xml.sax.SAXNotRecognizedException {}\n");
		buffer.append("	public void setProperty(String name, Object value) throws org.xml.sax.SAXNotSupportedException, org.xml.sax.SAXNotRecognizedException {}\n");
		buffer.append("	public void parse(String systemId) throws java.io.IOException, org.xml.sax.SAXException {}\n");
		buffer.append("	public void parse(org.xml.sax.InputSource input) throws java.io.IOException, org.xml.sax.SAXException {\n");
		buffer.append("		try {\n");
		buffer.append("			Attributes attributes = new AttributesImpl();\n");
		buffer.append("			this.fieldContentHandler.startDocument();\n");

		if (beanClassField.equals("fieldCustomer")) {

			buffer.append("		this.fieldContentHandler.startElement(\"\", \"number\", \"number\", attributes);\n");
			buffer.append("				this.fieldContentHandler.characters(" + beanClassField + ".getNumber().toCharArray(), 0, " + beanClassField + ".getNumber().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"number\", \"number\");\n");
			buffer.append("		this.fieldContentHandler.startElement(\"\", \"firstName\", \"firstName\", attributes);\n");
			buffer.append("			this.fieldContentHandler.characters(" + beanClassField + ".getFirstName().toCharArray(), 0, " + beanClassField + ".getFirstName().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"firstName\", \"firstName\");\n");
			buffer.append("		this.fieldContentHandler.startElement(\"\", \"lastName\", \"lastName\", attributes);\n");
			buffer.append("			this.fieldContentHandler.characters(" + beanClassField + ".getLastName().toCharArray(), 0, " + beanClassField + ".getLastName().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"lastName\", \"lastName\");\n");
		} else if (beanClassField.equals("fieldPurchaseOrder")) {
			buffer.append("		this.fieldContentHandler.startElement(\"\", \"orderNumber\", \"orderNumber\", attributes);\n");
			buffer.append("			this.fieldContentHandler.characters(" + beanClassField + ".getOrderNumber().toCharArray(), 0, " + beanClassField + ".getOrderNumber().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"orderNumber\", \"orderNumber\");\n");

			buffer.append("		this.fieldContentHandler.startElement(\"\", \"description\", \"description\", attributes);\n");
			buffer.append("			this.fieldContentHandler.characters(" + beanClassField + ".getDescription().toCharArray(), 0, " + beanClassField + ".getDescription().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"description\", \"description\");\n");

			buffer.append("		this.fieldContentHandler.startElement(\"\", \"partNumber\", \"partNumber\", attributes);\n");
			buffer.append("			this.fieldContentHandler.characters(" + beanClassField + ".getPartNumber().toCharArray(), 0, " + beanClassField + ".getPartNumber().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"partNumber\", \"partNumber\");\n");

		} else if (beanClassField.equals("fieldAddress")) {
			buffer.append("		this.fieldContentHandler.startElement(\"\", \"customerNumber\", \"customerNumber\", attributes);\n");
			buffer.append("			this.fieldContentHandler.characters(" + beanClassField + ".getCustomerNumber().toCharArray(), 0, " + beanClassField + ".getCustomerNumber().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"customerNumber\", \"customerNumber\");\n");

			buffer.append("		this.fieldContentHandler.startElement(\"\", \"city\", \"city\", attributes);\n");
			buffer.append("			this.fieldContentHandler.characters(" + beanClassField + ".getCity().toCharArray(), 0, " + beanClassField + ".getCity().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"city\", \"city\");\n");

			buffer.append("		this.fieldContentHandler.startElement(\"\", \"state\", \"state\", attributes);\n");
			buffer.append("			this.fieldContentHandler.characters(" + beanClassField + ".getState().toCharArray(), 0, " + beanClassField + ".getState().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"state\", \"state\");\n");

			buffer.append("		this.fieldContentHandler.startElement(\"\", \"street\", \"street\", attributes);\n");
			buffer.append("			this.fieldContentHandler.characters(" + beanClassField + ".getStreet().toCharArray(), 0, " + beanClassField + ".getStreet().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"street\", \"street\");\n");

			buffer.append("		this.fieldContentHandler.startElement(\"\", \"zipCode\", \"zipCode\", attributes);\n");
			buffer.append("			this.fieldContentHandler.characters(" + beanClassField + ".getZipCode().toCharArray(), 0, " + beanClassField + ".getZipCode().length());\n");
			buffer.append("		this.fieldContentHandler.endElement(\"\", \"zipCode\", \"zipCode\");\n");

		}

		buffer.append("			this.fieldContentHandler.endDocument();\n");
		buffer.append("		} catch (Exception e) {\n");
		buffer.append("			e.printStackTrace();\n");
		buffer.append("		}			\n");
		buffer.append("	}\n");
		buffer.append("	public void setContentHandler(org.xml.sax.ContentHandler handler) {\n");
		buffer.append("		this.fieldContentHandler = handler;\n");
		buffer.append("	}\n");
		buffer.append("}  \n");

	}

	private static final String importsGen = "import java.io.*;\n" + "import com.myeis.services.*;\n" + "import org.apache.wsif.providers.jca.WSIFFormatHandler_JCA;\n" +
	"import javax.xml.transform.Transformer;\n" + "import javax.xml.transform.TransformerFactory;\n" + "import javax.xml.transform.sax.SAXSource;\n" +
	"import org.xml.sax.XMLReader;\n" +
	"import org.xml.sax.Attributes;\n" + "import org.xml.sax.ContentHandler;\n" + "import org.xml.sax.InputSource;\n" + "import org.xml.sax.helpers.AttributesImpl;\n\n\n\n";

	private static final String defaultMethodsGen =
		"public Object getElement(String elementName) {return null;}\n"
			+ "public void setElement(String elementName, Object element) {}\n"
			+ "public Object getElement(String name, int index) {return null;}\n"
			+ "public void setElement(String name, int index, Object value) {}\n"
			+ "public javax.xml.namespace.QName getPartQName(){\n"
			+ "        return null;\n"
			+ "    }\n"
			+ "    \n"
			+ "    public void setPartQName(javax.xml.namespace.QName qname){\n"
			+ "    }\n"
			+ "public void setInteractionSpec(javax.resource.cci.InteractionSpec arg0) {\n"
			+ "        return;\n"
			+ "    }\n";
	/**
	 * @see org.apache.wsif.providers.jca.toolplugin.FormatHandlerGenerator#createGenerationContext()
	 */
	public Object createGenerationContext() {
		return null;
	}
}